/*
	Think of these things as arrays with less potential memory problems.
*/

#ifndef FLOATSTRUCTS_H
#define FLOATSTRUCTS_H


#include <windows.h>//just need it so i can include GL
#include <gl\gl.h>
#include <math.h>
#include <fstream>

//float16 needs to know about nehe's identity function, but the rest of nehe's matrix
//functions need to know about float structs.  Thus the following prototype.
namespace gl_ex { void identity4x4(float *); }

namespace sven {

using namespace std;


class float16 {
public:
	float m[16];
	float16() { gl_ex::identity4x4(m); }
	float &operator[](const int i) { return m[i]; }
	const float &operator[](const int i) const { return m[i]; }
};

//not much in this one, right now only used to hold material color info
class float4 {
public:
	GLfloat v[4];
	GLfloat &operator[](const int i) { return v[i]; }
	const GLfloat &operator[](const int i) const { return v[i]; }
	float4() {}
	float4(float v0, float v1, float v2, float v3) { v[0]=v0; v[1]=v1; v[2]=v2; v[3]=v3; }

	friend float4 operator+(const float4 &a, const float4 &b) {
		return float4(
			a[0]+b[0],
			a[1]+b[1],
			a[2]+b[2],
			a[3]+b[3]);
	}
	friend float4 operator-(const float4 &a, const float4 &b) {
		return float4(
			a[0]-b[0],
			a[1]-b[1],
			a[2]-b[2],
			a[3]-b[3]);
	}
	friend float4 operator/(const float4 &a, const float d) {
		return float4(
			a[0]/d,
			a[1]/d,
			a[2]/d,
			a[3]/d);
	}
	friend float4 operator*(const float4 &a, const float d) {
		return float4(
			a[0]*d,
			a[1]*d,
			a[2]*d,
			a[3]*d);
	}


};

struct float3 {
	float v[3];
	float3(float * u) { v[0]=u[0]; v[1]=u[1]; v[2]=u[2]; }
	float3() { v[0]=v[1]=v[2]=0; }
	float3(float v0, float v1, float v2) { v[0]=v0; v[1]=v1; v[2]=v2; }
	float &operator[](const int i) { return v[i]; }
	const float &operator[](const int i) const { return v[i]; }
	void set(float x, float y, float z) {
		v[0]=x;v[1]=y;v[2]=z;
	}

	const bool operator == ( const float3& a ) const {
		return (v[0]==a[0] && v[1]==a[1] && v[2]==a[2]);
	}

	float3& operator += (const float3& f3) {
	   v[0] += f3[0];   v[1] += f3[1];   v[2] += f3[2];
	   return *this;
	}
	float3& operator -= (const float3& f3) {
	   v[0] -= f3[0];   v[1] -= f3[1];   v[2] -= f3[2];
	   return *this;
	}

	float3& operator *= (const float& s) {
	   v[0] *= s;   v[1] *= s;   v[2] *= s;
	   return *this;
	}

	void inc(float s, float i, float j, float k) {
		v[0]+=s*i; v[1]+=s*j; v[2]+=s*k;
	}

	void inc(float s, float3 vec) { inc(s,vec[1],vec[2],vec[3]); }

	//lets me do '-f3'
	float3 operator - () const {
		float3 o;
		o[0]=-v[0];o[1]=-v[1];o[2]=-v[2];
		return o;
	}

}; 

//inline vec3::vec3(float3 f) { el[0]=f[0]; el[1]=f[1]; el[2]=f[2]; }

struct float2 {
	float x,y;
	float2() {}
	float2(float x, float y) : x(x), y(y) {}
	float &operator [](int i)		{ return i ? y : x; }
	float  operator [](int i) const	{ return i ? y : x; }
};

ostream& operator<<(ostream &os, float2 bob);

ostream& operator<<(ostream &os, float3 bob);

ostream& operator<<(ostream &os, float16 foo);

void inMatrix(float * m, ifstream &in);
void inFloat16(float16 &m, ifstream &in);
void inFloat3(float3 &vec, ifstream &in);
void inFloat2(float2 &point, ifstream &in);

//////////////////////////
//Some mathy functions

//returns a X b
float3 cross(float3 a, float3 b);

//returns a . b
float dot(float3 a, float3 b);

float mag2(float3 m);
float mag(float3 m);
float3 normalize(float3 p);
inline float3 abs(float3 v) { 
	return float3(1,2,3);
	//float3 r(abs(v[0]),abs(v[1]),abs(v[2])); 
	//return r; 
}

float16 operator *(float16 a, float16 b);


float3 operator *(float16 a, float3 b);
float3 operator +(const float3 a, const float3 b);
float3 operator -(const float3 a, const float3 b);
float3 operator *(float3 m, float c);
float3 operator *(float c, float3 m);

inline float dist(float3 a, float3 b) {
	return sqrt(
		(a.v[0]-b.v[0])*(a.v[0]-b.v[0])+
		(a.v[1]-b.v[1])*(a.v[1]-b.v[1])+
		(a.v[2]-b.v[2])*(a.v[2]-b.v[2]));
}

}

#endif