//A library containing alot of the general code from Nehe's tutorials that I use.
//All praise http://nehe.gamedev.net/ !!!

#ifndef NEHE_H
#define NEHE_H

#include <nehe_math.h>

namespace nehe {

using namespace sven;
using namespace std_ex;

//someone else is responsible for defining all of these
//externs (not nehe.cpp).  If you are using SGG, then
//sgg.cpp takes care of it.
extern GLfloat PERSP_NEAR;
extern GLfloat PERSP_FAR;
extern GLfloat PERSP_ANGLE;

void setWndProc(WNDPROC w);

//so MSVC gave me some wierd bullshit about not supporting
//default template arguments (or even templating overloading apparently).
//So I have to do this cursor specification stuff the old C way.
HCURSOR Load_arrow_ani();
HCURSOR Load_IDC_ARROW();
HCURSOR Load_IDC_CROSS();

typedef HCURSOR (* HCURSOR_fun )();
BOOL CreateGLWindow(char* title, int width, int height, int bits, HCURSOR_fun load_cursor=Load_arrow_ani);

GLvoid KillGLWindow(GLvoid); 

GLvoid ReSizeGLScene(GLsizei width, GLsizei height);



struct NeheFont {
	string name;
	int height;
	NeheFont(string name, int height) : name(name), height(height) {}

	friend bool operator <(const NeheFont & a, const NeheFont & b) { 
		return a.height<b.height; 
	}

	friend bool operator ==(const NeheFont & a, const NeheFont & b) { 
		return (a.height==b.height) && (a.name==b.name); 
	}

	//null is defined as ("",0).
	//it is occationally useful as a flag value.
	const static NeheFont null;

	void operator=(const NeheFont & b) {
		name=b.name;
		height=b.height;
	}

};

const NeheFont standard_fonts[] = {
	NeheFont("Times New Roman",12),
	NeheFont("Arial",24),
	NeheFont("Verdana",24)
};



GLvoid BuildFonts(const NeheFont font_list[]=standard_fonts);
GLvoid KillFonts();

struct TextFormatCommon {
	float x, y;
	NeheFont font;
	rgba color;

	TextFormatCommon(float x=100.0f, float y=100.0f, 
		rgba color= rgba(),
		NeheFont font=NeheFont::null ) :
	x(x), y(y), font(font), color(color) {}
};

struct TextFormat : public TextFormatCommon {
	GLfloat ySpace;
	TextFormat(float x=100.0f, float y=100.0f, 
		rgba color= rgba(),
		NeheFont font=NeheFont::null, 
		GLfloat ySpace=.16) :
	ySpace(ySpace), TextFormatCommon(x,y,color,font) {}
};

struct TextFormat_px : public TextFormatCommon {
	GLfloat yScale;
	TextFormat_px(float x=100.0f, float y=100.0f, 
		rgba color= rgba(),
		NeheFont font=NeheFont::null, 
		GLfloat yScale=1.3) :
	yScale(yScale), TextFormatCommon(x,y,color,font) {}

};

void glPrintf(TextFormat info, const char *fmt, ...);
void glPrintf_px(TextFormat_px info, const char *fmt, ...);

//using a class allows the Kill functions to be implicit in the destructor
struct NeheGLWindow {
	NeheGLWindow(char* title, int width, int height, int bits, HCURSOR_fun load_cursor=Load_arrow_ani) {
		CreateGLWindow(title, width, height, bits, load_cursor);
		BuildFonts();
	}
	~NeheGLWindow() { KillFonts(); KillGLWindow();  }
};

//AUX_RGBImageRec *LoadBMP(char *Filename);

typedef enum {
	MIP_MAPPED,
	SIMPLE,
	TRANS_BLACK,
	MIP_TRANS_BLACK
} TextureType;

namespace nehe_error {
	struct not_in_render_mode {};
	struct image_open_failed : public string { 
		image_open_failed(string s) : string(s) {}
	};
};

void LoadGLTexture(const char * fname, GLuint tex, TextureType=MIP_MAPPED);



//used to get reactions from clicking on text.
//void neheFontBox(int x, int y, int width, NeheFont text);
/*
//nehe's origional printing function (more or less)
GLvoid glPrint(NeheFont font, const char *fmt, ...);

//glPrint2 is to glPrint as atan2 is to atan.
//glPrint2 will correctly process newlines, and takes x and y
//coordinates in pixel locations.
//yScale specifices spaceing between lines.
//yScale=1 will produce very cramped text, 1.2 is more comfortable.
void glPrint2(NeheFont font, float yScale, float x, float y, const char *fmt, ...);
*/
//a simplified version of glPrintf2



void standard_text_setup(TextFormat info, GLuint &font,int&h);

//pretty standard resource wrapper.
//note that the implicit assignment operator and copy constructor _do_not_
//generate new textures. 
//dangerous to use because any call to the destructor kills the texture.
/*class GLtexture {
	GLuint texture;
	
	//trying to make copies of GLtextures leads to bugs because of
	//the auto texture free in the deconstructor.
	GLtexture &operator=(const GLtexture &t) { return *this;}
	GLtexture(const GLtexture &t) {}

public:	

	GLtexture(const char * fname) {
		glGenTextures(1,&texture);
		LoadGLTexture(fname,texture);
	}
	virtual ~GLtexture() {
		glDeleteTextures(1,&texture);
	}
	operator GLuint() const {
		return texture;
	}
};*/

//better texture wrapping function.
//this one keeps a mapping of strings to GLuints, so
//every texture you use is initilized and deleted once, and only once.
//There is no danger in copying things around.
//the only problem is that neheTextures are not freed until the end of
//the program (i am debating whether or not to include an explicit kill
//function)
class neheTexture {
	GLuint texture;
	struct texture_set : public map<string,GLuint> {
		~texture_set() {
			for(iterator i=begin();i!=end();i++) 
				glDeleteTextures(1,&(i->second)); 
		}
	};
	static texture_set textures;
public:

	neheTexture() {}

	neheTexture(string fname) {
		texture_set::iterator t=textures.find(fname);
		if(t==textures.end()) {
			glGenTextures(1,&texture);
			LoadGLTexture(fname.c_str(),texture);
			textures[fname]=texture;
		}
		else {
			texture=t->second;
		}
	}

	operator GLuint() const {
		return texture;
	}
};

}




#endif