#include "std_precomp.h"

//-----------------------------------------------------------------------
//  Main Window
//-------------------------------------------------------------------------

wxMenu * FluidMenu();
wxMenu * UIMenu();
wxMenu * NPRMenu();
wxMenu * ShowMenu();


BEGIN_EVENT_TABLE(MyFrame, wxFrame)

	EVT_SIZE (MyFrame::OnSize)
    
	

	//file menu
	EVT_MENU(wxID_EXIT, MyFrame::OnExit)
	EVT_MENU(ID_PRINT_SCREEN, MyFrame::OnPrint)
	EVT_MENU(ID_RESIZE, MyFrame::OnResize)
	EVT_MENU(ID_BATCH_PRINT, MyFrame::OnBatchPrint)


	//fluid menu
	EVT_MENU(ID_SOLVER_PARAMS, MyFrame::SolverParams)
	EVT_MENU(ID_HAULT_SIM, MyFrame::HaultSim)
	EVT_MENU(ID_SET_BOUND_COLOR, MyFrame::BoundColor)

	//UI menu
	EVT_MENU(ID_STAM_UI, MyFrame::UI_Stam)
	EVT_MENU(ID_BOUNDS_UI, MyFrame::UI_Bounds)


	//display menu	
	EVT_MENU(SHOW_POINTS, MyFrame::ShowPoints)
	EVT_MENU(SHOW_TRIANGLES, MyFrame::ShowTriangles)
	EVT_MENU(SHOW_BRUSHES, MyFrame::ShowBrushes)
	EVT_MENU(SHOW_DENSITY, MyFrame::ShowDensity)
	EVT_MENU(SHOW_VELOCITY, MyFrame::ShowVelocity)
	EVT_MENU(SHOW_BOUNDS, MyFrame::ShowBounds)	


	//npr menu
	EVT_MENU(NPR_INIT_POINTS, MyFrame::NPR_InitPoints)
	EVT_MENU(ID_NPR_PARAMS, MyFrame::NPR_Params)
	EVT_MENU(ID_NPR_BCOLORS, MyFrame::NPR_BColors)
	EVT_MENU(ID_NRAND_PARAMS, MyFrame::NPR_RandParams)




END_EVENT_TABLE()


// My frame constructor
MyFrame::MyFrame(wxFrame *frame, const wxString& title, const wxPoint& pos,
                 const wxSize& size, long style)
         : wxFrame(frame, -1, title, pos, size, style)
{
    m_canvas = NULL;
	CreateStatusBar();
}

// Intercept menu commands
void MyFrame::OnExit(wxCommandEvent& event)
{
    Destroy();
}

bool hault;
void MyFrame::HaultSim(wxCommandEvent& event) {
	hault=!hault;
}


void MyFrame::OnPrint(wxCommandEvent& event) {    
	wxFileDialog dialog(this,
                        _T("Save screen to what file?"),
                        _T(""),
                        _T("screenshot.bmp"),
                        _T("Bitmap files (*.bmp)|*.bmp"),
                        wxSAVE);

    if (dialog.ShowModal() == wxID_OK)
			m_canvas->gl_print_screen(dialog.GetPath());

}

void MyFrame::OnBatchPrint(wxCommandEvent& event) {    
	
	//fun with modal dialogs
	wxFileDialog dialog(this,
                        _T("Save screen to what files?"),
                        _T(""),
                        _T("screenshot#.bmp"),
                        _T("Bitmap files (*.bmp)|*.bmp"),
                        wxSAVE);

    if (dialog.ShowModal() == wxID_OK) {

		long res = wxGetNumberFromUser(_T("How many images do you want to produce?"),
									   _T("(Min 1, Max 500)"), _T("Preparing Batch Print-Screen"),
										50, 1, 500, this );

		wxString msg;
		int icon;
		if ( res == -1 )
		{
			msg = _T("Invalid number entered or dialog cancelled.");
			icon = wxICON_HAND;
		}
		else
		{
			msg.Printf(_T("Batch Print will create %lu images."), res );
			icon = wxICON_INFORMATION;
		}

		m_canvas->batch_printing=true;
		m_canvas->batch_print_size=res;
		m_canvas->batch_prints=0;
		m_canvas->batch_print_fname=dialog.GetPath();

		SetStatusText("Batch Printing.....");
	}

}

void MyFrame::OnResize(wxCommandEvent& event) {
		long width = wxGetNumberFromUser(_T("Width"),
									   _T("(Min 1, Max 800)"), _T("Resize"),
										50, 1, 800, this );
		long height = wxGetNumberFromUser(_T("Height"),
									   _T("(Min 1, Max 800)"), _T("Resize"),
										50, 1, 800, this );

	SetSize(width,height);
}



/*------------------------------------------------------------------
  Application object ( equivalent to main() )
------------------------------------------------------------------ */

IMPLEMENT_APP(MyApp)


bool MyApp::OnInit(void)
{
  wxLog::SetTraceMask(wxTraceMessages);

  // Create the main frame window
  MyFrame *frame = new MyFrame(NULL, "A Semi-Lagrangian Fluid Simulation", wxPoint(50, 50),
                               wxSize(640, 480));
  // Give it an icon
#ifdef wx_msw
  frame->SetIcon(wxIcon("mondrian"));
#endif

  // Make a menubar
  wxMenuBar *menuBar = new wxMenuBar;

  wxMenu *fileMenu = new wxMenu;
  fileMenu->Append(ID_RESIZE, "&Resize Screen\tCtrl-r");
  fileMenu->Append(ID_PRINT_SCREEN, "&Print Screen\tCtrl-p", "Take a screen shot");
  fileMenu->Append(ID_BATCH_PRINT, "&Batch Print\tAlt-p","Take a bunch of screen shots (usefull for creating animations)");
  fileMenu->Append(wxID_EXIT, "&Quit","Exit the program\tCtrl-q");
  menuBar->Append(fileMenu, "&File");

  //menuBar->Append(StandardEmitterMenu(), "&Tweak");

  menuBar->Append(FluidMenu(), "&Fluid");
  menuBar->Append(NPRMenu(), "&NPR");
  menuBar->Append(UIMenu(), "&UI");
  menuBar->Append(ShowMenu(), "&Display");

//  menuBar->Append(ShowMenu(), "&Show");

  frame->SetMenuBar(menuBar);

  frame->m_canvas = new TestGLCanvas(frame, -1, wxDefaultPosition, wxDefaultSize);

  // Show the frame
  frame->Show(TRUE);

  //frame->ShowFullScreen(1);

  return TRUE;
}

//something silly happens with readbuffer if the dimensions
//of the window are not multiples of 4
void MyFrame::OnSize(wxSizeEvent &event) {
	int w=event.GetSize().GetWidth(),h=event.GetSize().GetHeight();
	int w4=(w/4)*4, h4=(h/4)*4;
    if(w!=w4 || h!=h4) { 
		wxFrame::SetSize(w4,h4); 
	}
	else {
		wxFrame::OnSize(event);
	}
}